require "import"
import "android.speech.tts.TextToSpeech"
import "java.util.*"
import "android.speech.RecognizerIntent"

local tts
local context = activity or service

function initTTS()
    if context then
        tts = TextToSpeech(context, TextToSpeech.OnInitListener {
            onInit = function(status)
                if status == TextToSpeech.SUCCESS then
                    tts.setLanguage(Locale("ro"))
                    check_update()
                end
            end
        })
    else
        print("Contextul nu este disponibil, încerc din nou...")
        os.execute("sleep 1")
        initTTS()
    end
end

function check_update()
    local update_handle = io.popen("dumpsys update_engine | grep 'UpdateAvailable'")
    local update_result = update_handle:read("*a") or ""
    update_handle:close()

    local last_update_handle = io.popen("getprop ro.build.date.utc")
    local last_update_result = last_update_handle:read("*a") or ""
    last_update_handle:close()

    local version_handle = io.popen("getprop ro.build.version.release")
    local version_result = version_handle:read("*a") or ""
    version_handle:close()

    last_update_result = last_update_result:gsub("\n", "")
    version_result = version_result:gsub("\n", "")

    local months = {
        ["Jan"] = "ianuarie", ["Feb"] = "februarie", ["Mar"] = "martie", ["Apr"] = "aprilie",
        ["May"] = "mai", ["Jun"] = "iunie", ["Jul"] = "iulie", ["Aug"] = "august",
        ["Sep"] = "septembrie", ["Oct"] = "octombrie", ["Nov"] = "noiembrie", ["Dec"] = "decembrie"
    }

    local timestamp = tonumber(last_update_result)
    local formatted_date = "necunoscută"

    if timestamp and timestamp > 1000000000 then
        local timeTable = os.date("*t", timestamp)
        formatted_date = timeTable.day .. " " .. months[os.date("%b", timestamp)] .. " " .. timeTable.year
    end

    local message = "Ultima actualizare de sistem a fost pe " .. formatted_date .. ". Versiunea actuală Android este " .. version_result .. ". "

    if update_result:match("true") then
        message = message .. "Este disponibilă o actualizare de software. Dacă dorești să actualizezi telefonul, rostește 'Actualizare telefon'."
        speak_message(message, true)
    else
        message = message .. "Nu există actualizări disponibile."
        speak_message(message, false)
    end
end

function speak_message(message, listen)
    if tts then
        tts.speak(message, TextToSpeech.QUEUE_FLUSH, nil, nil)
        if listen then
            os.execute("sleep 4")
            listen_for_update_command()
        end
    else
        print("TTS nu este inițializat.")
    end
end

function listen_for_update_command()
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ro")
    intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Rostește 'Actualizare telefon' pentru a începe actualizarea")

    context.startActivityForResult(intent, 1)
end

function onActivityResult(requestCode, resultCode, data)
    if requestCode == 1 and resultCode == -1 then
        local results = data:getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
        local spokenText = results:get(0)

        if spokenText:lower():find("actualizare telefon") then
            os.execute("am start -a android.intent.action.SYSTEM_UPDATE_SETTINGS")
        else
            print("Comanda vocală nu a fost recunoscută.")
        end
    end
end

function onDestroy()
    if tts then
        tts.stop()
        tts.shutdown()
    end
end

initTTS()